from typing import List

from livestyled.models.venue import Venue


class FulfilmentPointCategoryTranslation:
    def __init__(
            self,
            id,
            language,
            title
    ):
        self.id = id
        self.language = language
        self.title = title


class FulfilmentPointTranslation:
    def __init__(
            self,
            language,
            title,
            description,
            collection_note
    ):
        self.language = language
        self.title = title
        self.description = description
        self.collection_note = collection_note


class FulfilmentPointCategory:
    def __init__(
            self,
            id,
            status,
            translations,
    ):
        self.id = id
        self.status = status

        if translations:
            for translation in translations:
                self.translations = []
                if isinstance(translation, FulfilmentPointCategoryTranslation):
                    self.translations.append(translation)
                elif isinstance(translation, dict):
                    self.translations.append(FulfilmentPointCategoryTranslation(**translation))
        else:
            self.translations = []

    @classmethod
    def placeholder(
            cls,
            id
    ):
        return cls(
            id=id,
            status=None,
            translations=None
        )


class FulfilmentPoint:
    def __init__(
            self,
            id,
            status,
            image_url,
            map_image_url,
            lat,
            long,
            type,
            position,
            reference,
            translations,
            categories,
            venue,
            external_id,
    ):
        self.id = id
        self.status = status
        self.image_url = image_url
        self.map_image_url = map_image_url
        self.lat = lat
        self.long = long
        self.type = type
        self.position = position
        self.reference = reference
        self.external_id = external_id

        if translations:
            for translation in translations:
                self.translations = []
                if isinstance(translation, FulfilmentPointTranslation):
                    self.translations.append(translation)
                elif isinstance(translation, dict):
                    self.translations.append(FulfilmentPointTranslation(**translation))
        else:
            self.translations = []

        if categories:
            for category in categories:
                self.categories = []
                if isinstance(category, FulfilmentPointCategory):
                    self.categories.append(category)
                elif isinstance(category, dict):
                    self.categories.append(FulfilmentPointCategory(**category))
                elif isinstance(category, int):
                    self.categories.append(FulfilmentPointCategory.placeholder(id=category))
        else:
            self.categories = []

        if venue:
            if isinstance(venue, Venue):
                self.venue = venue
            elif isinstance(venue, dict):
                self.venue = Venue(**venue)
            elif isinstance(venue, (str, int)):
                self.venue = Venue.placeholder(id=venue)
        else:
            self.venue = None

    @classmethod
    def placeholder(cls, id):
        return cls(
            id,
            status=None,
            image_url=None,
            map_image_url=None,
            lat=None,
            long=None,
            type=None,
            position=None,
            reference=None,
            translations=None,
            categories=None,
            venue=None,
            external_id=None
        )

    @classmethod
    def create_new(
            cls,
            external_id,
            status,
            type: str,
            position: int,
            reference: str,
            image_url: str or None = None,
            map_image_url: str or None = None,
            lat: int or None = None,
            long: int or None = None,
            translations: List or None = None,
            categories: List or None = None,
            venue: str or None = None
    ):
        fulfilment_point = FulfilmentPoint(
            id=None,
            external_id=external_id,
            status=status,
            position=position,
            image_url=image_url,
            map_image_url=map_image_url,
            lat=lat,
            long=long,
            type=type,
            reference=reference,
            translations=translations,
            categories=categories,
            venue=venue
        )
        return fulfilment_point

    def diff(self, other):
        differences = {}
        fields = (
            'external_id', 'type', 'status', 'position', 'reference', 'image_url', 'map_image_url', 'lat', 'long',
            'translations', 'categories', 'venue'
        )
        for field in fields:
            if getattr(self, field) != getattr(other, field):
                differences[field] = getattr(self, field)
        return differences



from typing import List

from livestyled.models.fulfilment_point import FulfilmentPoint


class ProductVariant:
    def __init__(
            self,
            id,
            price,
            stocks,
            external_id,
            translations,
            product,
    ):
        self.id = id
        self.price = price
        self.stocks = stocks
        self.external_id = external_id
        self.translations = translations
        if product:
            if isinstance(product, Product):
                self.product = product
            elif isinstance(product, int):
                self.product = Product.placeholder(id=product)
            elif isinstance(product, dict):
                self.product = Product(**product)
        else:
            self.product = None

    @classmethod
    def placeholder(cls, id):
        return cls(
            id,
            price=None,
            stocks=None,
            external_id=None,
            translations=None,
            product=None
        )

    @classmethod
    def create_new(
            cls,
            external_id,
            price,
            product: 'Product',
            translations,
            stocks
    ):
        product_variant = ProductVariant(
            id=None,
            external_id=external_id,
            price=price,
            translations=translations,
            product=product,
            stocks=stocks
        )

        return product_variant

    def diff(self, other):
        differences = {}
        fields = (
            'external_id', 'price', 'translations', 'product'
        )
        for field in fields:
            if getattr(self, field) != getattr(other, field):
                differences[field] = getattr(self, field)
        return differences


class ProductTranslation:
    def __init__(
            self,
            id,
            language,
            title,
            description,
            created_at,
            updated_at
    ):
        self.id = id
        self.language = language
        self.title = title
        self.description = description
        self.created_at = created_at
        self.updated_at = updated_at


class ProductModifierListTranslation:
    def __init__(
            self,
            id,
            language,
            title,
            created_at,
            updated_at
    ):
        self.id = id
        self.language = language
        self.title = title
        self.created_at = created_at
        self.updated_at = updated_at


class ProductImage:
    def __init__(
            self,
            id,
            product,
            position,
            created_at,
            updated_at,
            external_id,
            image_url
    ):
        self.id = id
        self.position = position
        self.created_at = created_at
        self.updated_at = updated_at
        self.external_id = external_id
        self.image_url = image_url
        self.product = None
        if product:
            if isinstance(product, Product):
                self.product = product
            elif isinstance(product, dict):
                self.product = Product(**product)
            elif isinstance(product, int):
                self.product = Product.placeholder(id=product)


class ProductCategory:
    def __init__(
            self,
            id,
            reference,
            position,
            external_id,
            status,
            translations
    ):
        self.id = id
        self.reference = reference
        self.position = position
        self.external_id = external_id
        self.status = status
        self.translations = translations

    @classmethod
    def create_new(
            cls,
            reference,
            position,
            external_id,
            status,
            translations,
    ):
        product_category = ProductCategory(
            id=None,
            reference=reference,
            position=position,
            external_id=external_id,
            status=status,
            translations=translations
        )

        return product_category

    def diff(self, other):
        differences = {}
        fields = (
            'external_id', 'status', 'reference', 'translations', 'position'
        )
        for field in fields:
            if getattr(self, field) != getattr(other, field):
                differences[field] = getattr(self, field)
        return differences


class Product:
    def __init__(
            self,
            id,
            status,
            reference,
            images,
            categories,
            translations,
            modifier_lists,
            external_id,
            holding_time,
            reconciliation_group,
            fulfilment_points,
            variants,
            core_product_category
    ):
        self.id = id
        self.status = status
        self.reference = reference
        self.modifier_lists = modifier_lists
        self.external_id = external_id
        self.holding_time = holding_time
        self.reconciliation_group = reconciliation_group
        self.core_product_category = core_product_category
        if images:
            self.images = []
            for image in images:
                if isinstance(image, ProductImage):
                    self.images.append(image)
                elif isinstance(image, dict):
                    image['product'] = self
                    self.images.append(ProductImage(**image))
        else:
            self.images = []

        if categories:
            self.categories = []
            for category in categories:
                if isinstance(category, ProductCategory):
                    self.categories.append(category)
                elif isinstance(category, dict):
                    self.categories.append(ProductCategory(**category))
        else:
            self.categories = []

        if translations:
            self.translations = []
            for translation in translations:
                if isinstance(translation, ProductTranslation):
                    self.translations.append(translation)
                elif isinstance(translation, dict):
                    self.translations.append(ProductTranslation(**translation))
        else:
            self.translations = []

        if fulfilment_points:
            self.fulfilment_points = []
            for fulfilment_point in fulfilment_points:
                if isinstance(fulfilment_point, FulfilmentPoint):
                    self.fulfilment_points.append(fulfilment_point)
                elif isinstance(fulfilment_point, dict):
                    self.fulfilment_points.append(FulfilmentPoint(**fulfilment_point))
                elif isinstance(fulfilment_point, int):
                    self.fulfilment_points.append(FulfilmentPoint.placeholder(id=fulfilment_point))

        if variants:
            self.variants = []
            for variant in variants:
                if isinstance(variant, ProductVariant):
                    self.variants.append(variant)
                elif isinstance(variant, dict):
                    variant['product'] = self
                    self.variants.append(ProductVariant(**variant))
                elif isinstance(variant, int):
                    self.variants.append(ProductVariant.placeholder(id=variant))

    @classmethod
    def placeholder(
            cls,
            id
    ):
        return cls(
            id=id,
            status=None,
            reference=None,
            images=None,
            categories=None,
            translations=None,
            external_id=None,
            variants=None,
            fulfilment_points=None,
            holding_time=None,
            reconciliation_group=None,
            modifier_lists=None,
            core_product_category=None
        )

    @classmethod
    def create_new(
            cls,
            external_id,
            status,
            reference,
            categories: List[ProductCategory] or None,
            translations,
            price,
            fulfilment_points: List[FulfilmentPoint] or None,
            core_product_category,
            holding_time: int or None,
            images,
            modifier_lists,
            reconciliation_group,
            variants: List[ProductVariant] or None
    ):
        product = Product(
            id=None,
            status=status,
            reference=reference,
            categories=categories,
            translations=translations,
            external_id=external_id,
            fulfilment_points=fulfilment_points,
            core_product_category=core_product_category,
            holding_time=holding_time,
            images=images,
            modifier_lists=modifier_lists,
            reconciliation_group=reconciliation_group,
            variants=variants,
        )

        return product

    def diff(self, other):
        differences = {}
        fields = (
            'external_id', 'status', 'reference', 'translations', 'categories', 'fulfilment_points',
            'core_product_category', 'holding_time', 'images', 'modifier_lists', 'reconciliation_group',
            'variants'
        )
        for field in fields:
            if getattr(self, field) != getattr(other, field):
                differences[field] = getattr(self, field)
        return differences



from datetime import datetime, timedelta, timezone
import os

from livestyled.models import Product, ProductImage, ProductTranslation
from livestyled.resource_client import LiveStyledResourceClient
from livestyled.tests.utils import configure_mock_responses

FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'fixtures')
TEST_API_DOMAIN = 'test.livestyled.com'
CONTENT_TYPE = 'application/ld+json'


def test_get_products(requests_mock):
    mock_responses = (
        ('GET', 'https://' + TEST_API_DOMAIN + '/v4/sell/products', 'mock_responses/ls_api/products.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    resource_client = LiveStyledResourceClient(TEST_API_DOMAIN, 'bar')
    products = list(resource_client.get_products())
    assert len(products) == 30
    products.sort(key=lambda o: o.id)
    product = products[2]

    assert isinstance(product, Product)
    assert product.id == 3
    assert product.status == 'ACTIVE'
    assert product.reference == "Men's Tank"
    assert len(product.images) == 1
    assert isinstance(product.images[0], ProductImage)
    assert product.images[0].id == 4
    assert product.images[0].position == 1
    assert product.images[0].created_at is None
    assert product.images[0].updated_at is None
    assert product.images[0].external_id == "6N6H3B3IZVGBK3OMFOCFNGE2"
    assert product.images[0].image_url == 'https://apiv3.s3.eu-west-1.amazonaws.com/productimage/original-2.jpeg'
    assert product.images[0].product.id == 3
    assert len(product.translations) == 1
    assert isinstance(product.translations[0], ProductTranslation)
    assert product.translations[0].id == 101138
    assert product.translations[0].title == "Men's Tank"
    assert product.translations[0].description is None
    assert product.translations[0].language == 'en'
    assert product.translations[0].created_at is None
    assert product.translations[0].updated_at is None



import json
import os

from livestyled.models import Device, DeviceReality, Reality
from livestyled.resource_client import LiveStyledResourceClient
from livestyled.tests.utils import configure_mock_responses

FIXTURES_DIR = os.path.join(os.path.dirname(__file__), 'fixtures')
TEST_API_DOMAIN = 'test.livestyled.com'
CONTENT_TYPE = 'application/ld+json'


def test_create_device_reality(requests_mock):
    mock_responses = (
        ('POST', 'https://' + TEST_API_DOMAIN + '/v4/user_management/device_realities', 'mock_responses/ls_api/user_management/new_device_reality.json', 200),
    )
    configure_mock_responses(requests_mock, mock_responses, FIXTURES_DIR, CONTENT_TYPE)

    resource_client = LiveStyledResourceClient(TEST_API_DOMAIN, 'bar')
    device_reality = DeviceReality.create_new(
        device=Device.placeholder(id=1),
        reality=Reality.placeholder(id=9),
        value='is this the real reality'
    )
    resource_client.create_device_reality(device_reality)

    assert len(requests_mock.request_history) == 1
    assert requests_mock.request_history[0].method == 'POST'
    assert requests_mock.request_history[0].url == 'https://test.livestyled.com/v4/user_management/device_realities'
    assert json.loads(requests_mock.request_history[0].body) == {
        'value': 'is this the real reality',
        'device': '/user_management/devices/1',
        'reality': '/user_management/realities/9'
    }
